# -*- coding:utf-8 -*-
#---------------------------------------------------------------------
#
# Visor Geogrfico
# Copyright (C) 2009 Germn Carrilo (GeoTux)
#
#---------------------------------------------------------------------
#
# licensed under the terms of GNU GPL 2
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software. If not, see <http://www.gnu.org/licenses/>.
#
#---------------------------------------------------------------------

from os.path import isfile

from PyQt4.QtCore import QFileInfo, QSettings, QVariant
from PyQt4.QtGui import QFileDialog, QMessageBox, QColor

from qgis.core import ( QGis, QgsDataSourceURI, QgsMapLayerRegistry,
                        QgsRasterLayer, QgsVectorLayer )

from dialogos.utilidades import Utilidades

class Cargue():
    """
        Clase Cargue:
        -------------
        Provee las herramientas para cargar informacin geogrfica al canvas.
        Maneja las variables layers y bSRSDefinido, la primera es la lista de
            capas del canvas y la segunda es un flag para conocer si ya ha sido
            definido un Sistema de Referencia en el mapa.

        parent: Widget padre. Sirve para posicionamiento de ventanas y para
                emitir seal de cambio de unidades del mapa.
        canvas: Widget para mostrar informacin geogrfica.
    """
    def __init__( self, parent, canvas ):
        self.parent = parent
        self.canvas = canvas

        self.bSRSDefinido = False # Flag: reproyectar o asignar proyeccin
        self.utils = Utilidades.obtenerInstancia()

    def cargarShapefile( self, layerPaths=None ):
        """ Agregar Shapefile al canvas """
        settings = QSettings()
        ultimaRutaAbrirShapefile = settings.value( 'Rutas/AbrirShapefile', QVariant('.') ).toString()

        if not layerPaths:
            layerPaths = QFileDialog.getOpenFileNames( self.parent, "Abrir Shapefile",
                ultimaRutaAbrirShapefile, "Shapefiles (*.shp *.SHP)" )
        for layerPath in layerPaths:
            layerInfo = QFileInfo( layerPath )
            settings.setValue( 'Rutas/AbrirShapefile', QVariant( layerInfo.absolutePath() ) )
            layerProvider = "ogr"

            if isfile( layerPath ):
                # Crear el layer
                layer = QgsVectorLayer( layerPath,
                    str( layerInfo.fileName() ).capitalize(), layerProvider )

                if layer.isValid():
                    layer.setProviderEncoding( "ISO8859-15" )
                    self.agregarCapa( layer )

    def cargarPostGIS( self, dictConexion=None ):
        """ Agregar capa de PostGIS al canvas """
        if not dictConexion:
            from dialogos.cargue_postgis import DialogoCarguePostGIS
            dialogoPg = DialogoCarguePostGIS.obtenerInstancia( self.parent )
            dialogoPg.mostrar()

            if dialogoPg.resultado:
                dictConexion = { 'servidor':dialogoPg.servidor, 'puerto':dialogoPg.puerto,
                    'bd':dialogoPg.bd, 'usuario':dialogoPg.usuario, 'contrasena':dialogoPg.contrasena,
                    'esquema':dialogoPg.esquema, 'tabla':dialogoPg.tabla, 'geom':dialogoPg.geom }

        if dictConexion:
            uri = QgsDataSourceURI()
            # Host, puerto, bd, usuario, password
            uri.setConnection( dictConexion[ 'servidor' ], dictConexion[ 'puerto' ],
                dictConexion[ 'bd' ], dictConexion[ 'usuario' ], dictConexion[ 'contrasena' ] )

            # Esquema, tabla, columna geometria y WHERE opcional
            uri.setDataSource( dictConexion[ 'esquema' ], dictConexion[ 'tabla' ],
                dictConexion[ 'geom' ] )

            layer = QgsVectorLayer( uri.uri(), dictConexion[ 'tabla' ],
                "postgres" )

            if layer.isValid():
                layer.setProviderEncoding( "ISO8859-15" )
                self.agregarCapa( layer )
            else:
                self.utils.mostrarMensaje( self.parent, 'Advertencia',
                    'No se ha podido establecer la conexin con \nla base de ' \
                    + 'datos que especificaste.\n\nRevisa los parmetros ' \
                    + 'de conexin.', QMessageBox.Warning )

    def cargarRaster( self, layerPaths=None ):
        """ Agregar Raster al canvas """
        settings = QSettings()
        ultimaRutaAbrirRaster = settings.value( 'Rutas/AbrirRaster', QVariant('.') ).toString()

        if not layerPaths:
            layerPaths = QFileDialog.getOpenFileNames( self.parent,
                "Abrir rster", ultimaRutaAbrirRaster,
                "GeoTIFF (*.tif *.TIF);;USGS ASCII DEM (*.dem *.DEM);;" \
                "Grid ASCII (*.asc *.ASC);;Erdas Imagine (*.img *.IMG);;" \
                "Portable Network Graphics (*.png *.PNG);;JPEG (*.jpg *.JPG);;" \
                "Graphics Interchange Format (*.gif *.GIF);;BMP (*.bmp *.BMP)" )

        for layerPath in layerPaths:
            layerInfo = QFileInfo( layerPath )
            settings.setValue( 'Rutas/AbrirRaster', QVariant( layerInfo.absolutePath() ) )

            if isfile( layerPath ):
                # Crear el layer
                layer = QgsRasterLayer( layerInfo.filePath(),
                    layerInfo.completeBaseName() )

                if layer.isValid():
                    self.agregarCapa( layer )

    def cargarWMS( self ):
        #url = 'http://mapbender.wheregroup.com/cgi-bin/mapserv?map=/data/umn/osm/osm_basic.map'
        #layers = [ 'Grenzen' ]
        #styles = [ 'default' ]
        #format = 'image/png'
        #crs = 'EPSG:4326'
        #rlayer = QgsRasterLayer( 0, url, 'Europa', 'wms', layers, styles,
            #format, crs )

        #url = 'http://wms.lizardtech.com/lizardtech/iserv/ows'
        #layers = [ 'MODIS' ]
        #styles = [ '' ]
        #format = 'image/png'
        #crs = 'EPSG:4326'
        #rlayer = QgsRasterLayer( 0, url, 'Modis', 'wms', layers, styles,
            #format, crs )

        #url = 'http://geoservice.igac.gov.co/OGCRaster?service=WMS&version=1.1.1'
        #layers = [ 'fondo' ]
        #styles = [ 'default' ]
        #format = 'image/png'
        #crs = 'EPSG:4326'
        #rlayer = QgsRasterLayer( 0, url, 'F', 'wms', layers, styles,
            #format, crs )

        url = 'http://ovc.catastro.meh.es/Cartografia/WMS/ServidorWMS.aspx'
        layers = [ 'Catastro' ]
        styles = [ 'default' ]
        format = 'image/png'
        crs = 'EPSG:4326'
        rlayer = QgsRasterLayer( 0, url, 'Catastro Spain', 'wms', layers, styles,
            format, crs )

        if rlayer.isValid():
            self.agregarCapa( rlayer )

    def agregarCapa( self, capa ):

        if ( capa.__class__ == QgsVectorLayer ):
            #Cambiar el color del layer
            symbols = capa.renderer().symbols()
            #symbol = symbols[ 0 ]

            #if capa.geometryType() == 0: # Punto
                #symbol.setFillColor( QColor.fromRgb( 244,186,11 ) )
                #symbol.setPointSize( 1 )
            #elif capa.geometryType() == 1: # Linea
                #symbol.setColor( QColor.fromRgb( 111,165,215 ) )
            #else:  #Poligono
                #symbol.setFillColor( QColor.fromRgb( 176,251,163 ) )
                #symbol.setColor( QColor.fromRgb( 152,157,162 ) )
        #elif ( capa.__class__ == QgsRasterLayer ):
            #if ( capa.rasterType() == QgsRasterLayer.Multiband ):
                #print "1"
                ##capa.setDrawingStyle(QgsRasterLayer.MultiBandSingleBandPseudoColor)
            #elif ( capa.rasterType() == QgsRasterLayer.Palette ):
                #print "2"
                ##capa.setDrawingStyle(QgsRasterLayer.PalettedSingleBandPseudoColor)
            #else: # QgsRasterLayer.GrayOrUndefined
                #print "3"
                #capa.setDrawingStyle( QgsRasterLayer.SingleBandGray )

        # Fijar el extent del mapa al del primer layer cargado y fijar SRS y unidades
        if self.canvas.layerCount() == 0:
           self.canvas.setExtent( capa.extent() )

        # Agregar el layer al registro
        QgsMapLayerRegistry.instance().addMapLayer( capa )

        # Validar el Sistema de Referencia Espacial de la capa
        if ( ( capa.extent().xMaximum() - capa.extent().xMinimum() > 400 ) \
            and capa.srs().geographicFlag() ):

            #self.utils.mostrarMensaje( self.parent, "Sistema de Referencia",
                #"No se ha podido cargar informacin del Sistema de Referencia" \
                #+ " de la capa que vas a agregar.\n\nPor favor defnelo a" \
                #+ " continuacin para que el mapa pueda reconocerlo " \
                #+ "correctamente.", QMessageBox.Warning )
            #codEpsg = self.utils.definirSRS( self.parent, "capa" )

            #if ( codEpsg ):
                #capa.setCrs( QgsCoordinateReferenceSystem( codEpsg,
                    #QgsCoordinateReferenceSystem.EpsgCrsId ) )
                #self.bSRSDefinido = True
            #else:
                #self.utils.mostrarMensaje( self.parent, "Sistema de Referencia",
                    #"No has definido un Sistema de Referencia adecuado para " \
                    #+ "la capa que vas a agregar.\n\nLa informacin de " \
                    #+ "coordenadas, escala y distancia puede estar " \
                    #+ "incorrecta.", QMessageBox.Critical )
                #self.bSRSDefinido = False
            self.bSRSDefinido = False
            print "No se ha encontrado SRS"
        else: # Si no hubo problemas se puede reproyectar
            self.bSRSDefinido = True

        if ( not capa.srs().geographicFlag() ): # Cambiar unidades del mapa
            self.canvas.setMapUnits( QGis.Meters )
            self.canvas.updateScale()
            # Emitir SIGNAL con las nuevas unidades como argumento
            self.parent.emitirSignalUnidadesMapaCambiadas()
