# -*- coding:utf-8 -*-
# Esta clase expone las funcionalidades de QgisApp y las
# hace disponibles para plugins.
# https://svn.osgeo.org/qgis/trunk/qgis/src/gui/qgisinterface.h
#---------------------------------------------------------------------
#
# Visor Geogrfico
# Copyright (C) 2009 Germn Carrilo (GeoTux)
#
#---------------------------------------------------------------------
#
# licensed under the terms of GNU GPL 2
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software. If not, see <http://www.gnu.org/licenses/>.
#
#---------------------------------------------------------------------

from PyQt4.QtCore import QObject, QString
from PyQt4.QtGui import QMenu

class QgisInterface( QObject ):
    """ Clase para exponer funcionalidades de QGis a los plugins """
    def __init__( self, visor, canvas ):
        QObject.__init__( self )
        self.miApp = visor
        self.canvas = canvas
        self.toolBarPlugins = None

    def zoomFull( self ):
        """ Zoom to the map full extent """
        self.canvas.zoomToFullExtent()

    def zoomToPrevious( self ):
        """ Zoom to previous view extent """
        self.canvas.zoomToPreviousExtent()

    def zoomToNext( self ):
        """ Zoom to next view extent """
        self.canvas.zoomToNextExtent()

    #//! Zoom to extent of the active layer
    #virtual void zoomToActiveLayer() = 0;

    #def addVectorLayer( vectorLayerPath, baseName, providerKey ):
        """ Add a vector layer """

    #def addRasterLayer( rasterLayerPath, baseName = QString() ):
        """ Add a raster layer given a raster layer file name """

    #def addRasterLayer( url, layerName, providerKey, layers,
                        #styles, format, crs ):
        """ Add a WMS layer """

    #//! Add a project
    #virtual bool addProject( QString theProject ) = 0;
    #//! Start a blank project
    #virtual void newProject( bool thePromptToSaveFlag = false ) = 0;

    def activeLayer( self ):
        """ Get pointer to the active layer (layer selected in the legend) """
        if self.miApp.activeLayer():
            return self.miApp.activeLayer().layer()
        return None

    def addToolBarIcon( self, qAction ):
        """ Add an icon to the plugins toolbar """
        if not self.toolBarPlugins:
            self.toolBarPlugins = self.addToolBar( "Plugins" )
        self.toolBarPlugins.addAction( qAction )

    def removeToolBarIcon( self, qAction ):
        """ Remove an action (icon) from the plugin toolbar """
        if not self.toolBarPlugins:
            self.toolBarPlugins = self.addToolBar( "Plugins" )
        self.toolBarPlugins.removeAction( qAction )

    def addToolBar( self, name ):
        """ Add toolbar with specified name """
        toolBar = self.miApp.addToolBar( name )
        toolBar.setObjectName( name )
        return toolBar

    def mapCanvas( self ):
        """ Return a pointer to the map canvas """
        return self.canvas

    def mainWindow( self ):
        """ Return a pointer to the main window (instance of QgisApp in case of QGIS) """
        return self.miApp

    #Return mainwindows / composer views of running composer instances (currently only one)
    #virtual QList<QgsComposerView*> activeComposers() = 0;

    def addPluginToMenu( self, name, action ):
        """ Add action to the plugins menu """
        menu = self.getPluginMenu( name )
        menu.addAction( action )

    def removePluginMenu( self, name, action ):
        """ Remove action from the plugins menu """
        menu = self.getPluginMenu( name )
        menu.removeAction( action )

    def addDockWidget( self, area, dockwidget ):
        """ Add a dock widget to the main window """
        self.miApp.addDockWidget( area, dockwidget )
        dockwidget.show()

        # refresh the map canvas
        self.canvas.refresh()

    #virtual void removeDockWidget( QDockWidget * dockwidget ) = 0;
        """ Remove specified dock widget from main window (doesn't delete it) """

    def refreshLegend( self, mapLayer ):
        """ Refresh the legend of a layer """
        self.miApp.refreshLayerSymbology( mapLayer )

    def pluginMenu( self ):
        """ Retorna el men principal de los plugins """
        return self.miApp.menuPlugins

    def getPluginMenu( self, menuName ):
        """ Retorna el men secundario que corresponde a un plugin """
        before = None

        if self.pluginMenu():
            actions = self.pluginMenu().actions()
            for action in actions:
                comp = QString( menuName ).localeAwareCompare( action.text() )
                if ( comp < 0 ):
                    before = action # Add item before this one
                    break
                elif ( comp == 0 ):
                    # Plugin menu item already exists
                    return action.menu()

        # It doesn't exist, so create
        menu = QMenu( menuName, self.pluginMenu() )
        if before:
            self.pluginMenu().insertMenu( before, menu )
        else:
            self.pluginMenu().addMenu( menu )
        return menu
