# -*- coding:utf-8 -*-
#---------------------------------------------------------------------
#
# Visor Geogrfico
# Copyright (C) 2009 Germn Carrilo (GeoTux)
#
#---------------------------------------------------------------------
#
# licensed under the terms of GNU GPL 2
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software. If not, see <http://www.gnu.org/licenses/>.
#
#---------------------------------------------------------------------


import base64
import math

from PyQt4.QtCore import QString
from PyQt4.QtGui import QMessageBox, QColor

from qgis.core import QGis
from qgis.gui import QgsGenericProjectionSelector


class Utilidades():
    """ Esta clase provee utilidades comunes a las dems """

    instancia = None # Variable para controlar la nica instancia

    class SingletonHelper:
        """ Provee un mtodo factory para el Singleton Utilidades """
        def __call__( self, *args, **kw ) :

            if Utilidades.instancia is None :
                object = Utilidades()
                Utilidades.instancia = object

            return Utilidades.instancia

    obtenerInstancia = SingletonHelper() # Mtodo para obtener la instancia

    def __init__( self ):
        if not Utilidades.instancia is None :
            raise RuntimeError, 'Solo se permite una instancia de esta clase!'

        # Diccionarios para almacenar colores de capas en simbologa por clases
        self.simbologiaClasesEsRampa = {}
        self.simbologiaClasesColorInicial = {}
        self.simbologiaClasesColorFinal = {}

    def mostrarMensaje( self, parent, titulo, desc, icono=None ):
        """ Muestra mensajes al usuario """
        msg = QMessageBox( parent )
        msg.setText( desc )
        msg.setWindowTitle( titulo )
        if icono:
            msg.setIcon( icono )
        msg.addButton( 'Cerrar', QMessageBox.RejectRole )
        msg.exec_()

    def mostrarMensajeConfirmacion( self, parent, titulo, desc ):
        """ Muestra mensajes al usuario """
        msg = QMessageBox( parent )
        msg.setText( desc )
        msg.setWindowTitle( titulo )
        msg.setIcon( QMessageBox.Question )

        botonSi = msg.addButton( 'Si', QMessageBox.YesRole )
        botonNo = msg.addButton( 'No', QMessageBox.NoRole )
        msg.exec_()

        if msg.clickedButton() == botonSi:
            return "si"
        else:
            return "no"

    def formatearNumero( self, number, precision=0, group_sep='.',
            decimal_sep=',' ):
        """
            Funcin:
            --------
            Formatea nmeros

            number: Nmero a formatear
            precision: Nmero de decimales
            group_sep: Separador de miles
            decimal_sep: Separador decimal
        """
        number = ( '%.*f' % ( max( 0, precision ), number ) ).split( '.' )
        integer_part = number[ 0 ]

        if integer_part[ 0 ] == '-':
            sign = integer_part[ 0 ]
            integer_part = integer_part[ 1: ]
        else:
            sign = ''

        if len( number ) == 2:
            decimal_part = decimal_sep + number[ 1 ]
        else:
            decimal_part = ''

        integer_part = list( integer_part )
        c = len( integer_part )

        while c > 3:
            c -= 3
            integer_part.insert( c, group_sep )

        return sign + ''.join( integer_part ) + decimal_part

    def formatearAGrados( self, numero ):
        """ Devuelve una cadena con los grados, min y seg de un decimal """
        signo = ''
        if numero < 0:
            numero = math.fabs( numero );
            signo = '-';

        gra = math.floor( numero )
        minu = math.floor( ( numero - gra ) * 60 )
        segu = math.floor( ( ( ( numero - gra ) * 60 ) - minu ) * 60 )

        return signo + "%.0f"%gra + ' ' + "%.0f"%minu + "' " \
            + "%.0f"%segu + "\""

    def encStr( self, string ):
        return base64.b64encode( string )

    def decStr( self, string ):
        cadena = QString( "" )
        for char in base64.b64decode( string ):
            cadena.append( char ) if ( not QString( char ).isEmpty() ) else None
        return cadena

    def definirSRS( self, parent, mensaje ):
        """
            Funcin:
            --------
            Muestra el dilogo para definir un Sistema de Referencia

            parent: Widget padre, sirve para centrar el dilogo.
            mensaje: Sirve para determinar qu mensaje se muestra como
                    encabezado en el dilogo.
            Devuelve un cdigo EPSG
        """
        srs = QgsGenericProjectionSelector( parent )
        if mensaje == "capa":
            srs.setMessage( "<h2>Definir Sistema de Referencia para esta " \
                + "capa</h2>A continuacin selecciona el Sistema de " \
                + "Referencia apropiado para tus datos, con lo que se podr " \
                + "disponer correctamente de la escala, las coordenadas y la " \
                + "distancia en el mapa. <br /><br />     Recuerda que en " \
                + "Colombia se utilizan los <i>EPSG</i> <b>21892</b> y " \
                + "<b>21897</b> para el elipsoide Internacional, el " \
                + "<b>3115</b> para el <i>MAGNA-SIRGAS</i> y el <b>4326</b> " \
                + "para el <i>WGS84</i>, entre otros." )
        else:
            srs.setMessage( "<h2>Definir Sistema de Referencia para el mapa" \
                + "</h2>A continuacin selecciona el Sistema de Referencia " \
                + "para el mapa. Ten en cuenta que esta accin afectar todas" \
                + " las capas cargadas en el mapa. <br /><br />     Recuerda " \
                + "que en Colombia se utilizan los <i>EPSG</i> <b>21892</b> y" \
                + " <b>21897</b> para el elipsoide Internacional, el " \
                + "<b>3115</b> para el <i>MAGNA-SIRGAS</i> y el <b>4326</b> " \
                + "para el <i>WGS84</i>, entre otros." )
        if srs.exec_():
            return srs.selectedEpsg()
        else:
            return None

    def isEqual( self, a, b ):
        """ Compara dos nmeros flotantes """
        return abs( a - b ) < 0.0001

    def randomColor( self, geometryType ):
        """ Retorna un color aleatorio con base en el tipo de geometra """
        color = QColor()
        import random

        if ( geometryType == QGis.Point ):
            hue = random.randrange( 0, 255 )
            saturation = random.randrange( 0, 255 )
            value = random.randrange( 100, 255 )
        elif geometryType == QGis.Line:
            hue = random.randrange( 0, 359 )
            saturation = random.randrange( 230, 255 )
            value = random.randrange( 50, 255 )
        elif geometryType == QGis.Polygon:
            hue = random.randrange( 0, 255 )
            saturation = random.randrange( 0, 200 )
            value = random.randrange( 100, 255 )

        color.setHsv( hue, saturation, value )
        return color

    def getSimbologiaClasesEsRampa( self, layerId ):
        """ Retorna un booleano para indicar si la capa tiene rampa de colores """
        if not ( layerId in self.simbologiaClasesEsRampa ):
            self.simbologiaClasesEsRampa[ layerId ] = False
        return self.simbologiaClasesEsRampa[ layerId ]

    def setSimbologiaClasesEsRampa( self, layerId, esRampa ):
        """ Define un booleano para indicar si la capa tiene rampa de colores """
        self.simbologiaClasesEsRampa[ layerId ] = esRampa

    def getSimbologiaClasesColorInicial( self, layerId ):
        """ Retorna el color inicial de la rampa para la capa """
        if not ( layerId in self.simbologiaClasesColorInicial ):
            self.simbologiaClasesColorInicial[ layerId ] = QColor( 'red' )
        return self.simbologiaClasesColorInicial[ layerId ]

    def setSimbologiaClasesColorInicial( self, layerId, color ):
        """ Define el color inicial de la rampa para la capa """
        self.simbologiaClasesColorInicial[ layerId ] =  color

    def getSimbologiaClasesColorFinal( self, layerId ):
        """ Retorna el color final de la rampa para la capa """
        if not ( layerId in self.simbologiaClasesColorFinal ):
            self.simbologiaClasesColorFinal[ layerId ] = QColor( 'green' )
        return self.simbologiaClasesColorFinal[ layerId ]

    def setSimbologiaClasesColorFinal( self, layerId, color ):
        """ Define el color final de la rampa para la capa """
        self.simbologiaClasesColorFinal[ layerId ] =  color
